@@ -7,12 +7,15 @@ from urlparse import urljoin
 categories = ['images']
 
 url = 'https://secure.flickr.com/'
-search_url = url+'search/?{query}'
+search_url = url+'search/?{query}&page={page}'
 results_xpath = '//div[@id="thumbnails"]//a[@class="rapidnofollow photo-click" and @data-track="photo-click"]'  # noqa
 
+paging = True
+
 
 def request(query, params):
-    params['url'] = search_url.format(query=urlencode({'q': query}))
+    params['url'] = search_url.format(query=urlencode({'q': query}),
+                                      page=params['pageno'])
     return params
 
 
