@@ -6,11 +6,13 @@ from json import loads
 categories = ['images']
 
 url = 'https://ajax.googleapis.com/'
-search_url = url + 'ajax/services/search/images?v=1.0&start=0&rsz=large&safe=off&filter=off&{query}'  # noqa
+search_url = url + 'ajax/services/search/images?v=1.0&start={offset}&rsz=large&safe=off&filter=off&{query}'  # noqa
 
 
 def request(query, params):
-    params['url'] = search_url.format(query=urlencode({'q': query}))
+    offset = (params['pageno'] - 1) * 8
+    params['url'] = search_url.format(query=urlencode({'q': query}),
+                                      offset=offset)
     return params
 
 
