@@ -26,11 +26,13 @@ import json
 import cStringIO
 import os
 
+from datetime import datetime, timedelta
+from itertools import chain
 from flask import (
     Flask, request, render_template, url_for, Response, make_response,
     redirect, send_from_directory
 )
-from flask.ext.babel import Babel
+from flask.ext.babel import Babel, gettext, format_date
 from searx import settings, searx_dir
 from searx.engines import (
     search as do_search, categories, engines, get_engines_stats,
@@ -39,6 +41,7 @@ from searx.engines import (
 from searx.utils import UnicodeWriter, highlight_content, html_to_text
 from searx.languages import language_codes
 from searx.search import Search
+from searx.autocomplete import backends as autocomplete_backends
 
 
 app = Flask(
@@ -55,6 +58,8 @@ babel = Babel(app)
 favicons = ['wikipedia', 'youtube', 'vimeo', 'soundcloud',
             'twitter', 'stackoverflow', 'github']
 
+cookie_max_age = 60 * 60 * 24 * 365 * 23  # 23 years
+
 
 @babel.localeselector
 def get_locale():
@@ -86,10 +91,26 @@ def get_base_url():
 
 
 def render(template_name, **kwargs):
-    global categories
-    kwargs['categories'] = ['general']
-    kwargs['categories'].extend(x for x in
-                                sorted(categories.keys()) if x != 'general')
+    blocked_engines = request.cookies.get('blocked_engines', '').split(',')
+
+    autocomplete = request.cookies.get('autocomplete')
+
+    if autocomplete not in autocomplete_backends:
+        autocomplete = None
+
+    nonblocked_categories = (engines[e].categories
+                             for e in engines
+                             if e not in blocked_engines)
+
+    nonblocked_categories = set(chain.from_iterable(nonblocked_categories))
+
+    if not 'categories' in kwargs:
+        kwargs['categories'] = ['general']
+        kwargs['categories'].extend(x for x in
+                                    sorted(categories.keys())
+                                    if x != 'general'
+                                    and x in nonblocked_categories)
+
     if not 'selected_categories' in kwargs:
         kwargs['selected_categories'] = []
         cookie_categories = request.cookies.get('categories', '').split(',')
@@ -98,9 +119,16 @@ def render(template_name, **kwargs):
                 kwargs['selected_categories'].append(ccateg)
         if not kwargs['selected_categories']:
             kwargs['selected_categories'] = ['general']
+
+    if not 'autocomplete' in kwargs:
+        kwargs['autocomplete'] = autocomplete
+
+    kwargs['method'] = request.cookies.get('method', 'POST')
+
     return render_template(template_name, **kwargs)
 
 
+@app.route('/search', methods=['GET', 'POST'])
 @app.route('/', methods=['GET', 'POST'])
 def index():
     """Render index page.
@@ -109,12 +137,16 @@ def index():
     """
 
     if not request.args and not request.form:
-        return render('index.html')
+        return render(
+            'index.html',
+        )
 
     try:
         search = Search(request)
     except:
-        return render('index.html')
+        return render(
+            'index.html',
+        )
 
     # TODO moar refactor - do_search integration into Search class
     search.results, search.suggestions = do_search(search.query,
@@ -148,6 +180,23 @@ def index():
             if engine in favicons:
                 result['favicon'] = engine
 
+        # TODO, check if timezone is calculated right
+        if 'publishedDate' in result:
+            if result['publishedDate'].replace(tzinfo=None)\
+               >= datetime.now() - timedelta(days=1):
+                timedifference = datetime.now() - result['publishedDate']\
+                    .replace(tzinfo=None)
+                minutes = int((timedifference.seconds / 60) % 60)
+                hours = int(timedifference.seconds / 60 / 60)
+                if hours == 0:
+                    result['publishedDate'] = gettext(u'{minutes} minute(s) ago').format(minutes=minutes)  # noqa
+                else:
+                    result['publishedDate'] = gettext(u'{hours} hour(s), {minutes} minute(s) ago').format(hours=hours, minutes=minutes)  # noqa
+            else:
+                result['pubdate'] = result['publishedDate']\
+                    .strftime('%a, %d %b %Y %H:%M:%S %z')
+                result['publishedDate'] = format_date(result['publishedDate'])
+
     if search.request_data.get('format') == 'json':
         return Response(json.dumps({'query': search.query,
                                     'results': search.results}),
@@ -182,6 +231,7 @@ def index():
         selected_categories=search.categories,
         paging=search.paging,
         pageno=search.pageno,
+        base_url=get_base_url(),
         suggestions=search.suggestions
     )
 
@@ -189,7 +239,40 @@ def index():
 @app.route('/about', methods=['GET'])
 def about():
     """Render about page"""
-    return render('about.html')
+    return render(
+        'about.html',
+    )
+
+
+@app.route('/autocompleter', methods=['GET', 'POST'])
+def autocompleter():
+    """Return autocompleter results"""
+    request_data = {}
+
+    if request.method == 'POST':
+        request_data = request.form
+    else:
+        request_data = request.args
+
+    # TODO fix XSS-vulnerability
+    query = request_data.get('q', '').encode('utf-8')
+
+    if not query:
+        return
+
+    completer = autocomplete_backends.get(request.cookies.get('autocomplete'))
+
+    if not completer:
+        return
+
+    results = completer(query)
+
+    if request_data.get('format') == 'x-suggestions':
+        return Response(json.dumps([query, results]),
+                        mimetype='application/json')
+    else:
+        return Response(json.dumps(results),
+                        mimetype='application/json')
 
 
 @app.route('/preferences', methods=['GET', 'POST'])
@@ -210,6 +293,8 @@ def preferences():
     else:
         selected_categories = []
         locale = None
+        autocomplete = ''
+        method = 'POST'
         for pd_name, pd in request.form.items():
             if pd_name.startswith('category_'):
                 category = pd_name[9:]
@@ -218,46 +303,55 @@ def preferences():
                 selected_categories.append(category)
             elif pd_name == 'locale' and pd in settings['locales']:
                 locale = pd
+            elif pd_name == 'autocomplete':
+                autocomplete = pd
             elif pd_name == 'language' and (pd == 'all' or
                                             pd in (x[0] for
                                                    x in language_codes)):
                 lang = pd
+            elif pd_name == 'method':
+                method = pd
             elif pd_name.startswith('engine_'):
                 engine_name = pd_name.replace('engine_', '', 1)
                 if engine_name in engines:
                     blocked_engines.append(engine_name)
 
-        resp = make_response(redirect('/'))
+        resp = make_response(redirect(url_for('index')))
 
         user_blocked_engines = request.cookies.get('blocked_engines', '').split(',')  # noqa
 
         if sorted(blocked_engines) != sorted(user_blocked_engines):
-            # cookie max age: 4 weeks
             resp.set_cookie(
                 'blocked_engines', ','.join(blocked_engines),
-                max_age=60 * 60 * 24 * 7 * 4
+                max_age=cookie_max_age
             )
 
         if locale:
-            # cookie max age: 4 weeks
             resp.set_cookie(
                 'locale', locale,
-                max_age=60 * 60 * 24 * 7 * 4
+                max_age=cookie_max_age
             )
 
         if lang:
-            # cookie max age: 4 weeks
             resp.set_cookie(
                 'language', lang,
-                max_age=60 * 60 * 24 * 7 * 4
+                max_age=cookie_max_age
             )
 
         if selected_categories:
             # cookie max age: 4 weeks
             resp.set_cookie(
                 'categories', ','.join(selected_categories),
-                max_age=60 * 60 * 24 * 7 * 4
+                max_age=cookie_max_age
             )
+
+            resp.set_cookie(
+                'autocomplete', autocomplete,
+                max_age=cookie_max_age
+            )
+
+        resp.set_cookie('method', method, max_age=cookie_max_age)
+
         return resp
     return render('preferences.html',
                   locales=settings['locales'],
@@ -266,6 +360,7 @@ def preferences():
                   language_codes=language_codes,
                   categs=categories.items(),
                   blocked_engines=blocked_engines,
+                  autocomplete_backends=autocomplete_backends,
                   shortcuts={y: x for x, y in engine_shortcuts.items()})
 
 
@@ -274,7 +369,10 @@ def stats():
     """Render engine statistics page."""
     global categories
     stats = get_engines_stats()
-    return render('stats.html', stats=stats)
+    return render(
+        'stats.html',
+        stats=stats,
+    )
 
 
 @app.route('/robots.txt', methods=['GET'])
@@ -293,7 +391,11 @@ def opensearch():
     # chrome/chromium only supports HTTP GET....
     if request.headers.get('User-Agent', '').lower().find('webkit') >= 0:
         method = 'get'
-    ret = render('opensearch.xml', method=method, host=get_base_url())
+
+    ret = render('opensearch.xml',
+                 opensearch_method=method,
+                 host=get_base_url())
+
     resp = Response(response=ret,
                     status=200,
                     mimetype="application/xml")
